@@ -20,7 +20,7 @@
  * @author Akinobu Lee
  * @date   Thu May 12 13:31:47 2005
  *
- * $Revision: 1.26 $
+ * $Revision: 1.29 $
  * 
  */
 /*
@@ -89,6 +89,17 @@ initialize_HMM(JCONF_AM *amconf, Jconf *jconf)
     hmminfo_free(hmminfo);
     return NULL;
   }
+  if (debug2_flag) {
+    HTK_HMM_Data *dtmp;
+    int i;
+    for (dtmp = hmminfo->start; dtmp; dtmp = dtmp->next) {
+      printf("***\nname: %s\n", dtmp->name);
+      for (i=0;i<dtmp->state_num;i++) {
+	if (dtmp->s[i] == NULL) continue;
+	printf("state %d: id=%d   %s\n", i + 1, dtmp->s[i]->id, (dtmp->s[i]->name) ? dtmp->s[i]->name : "");
+      }
+    }
+  }
 
   /* set multipath mode flag */
   if (amconf->force_multipath) {
@@ -104,8 +115,13 @@ initialize_HMM(JCONF_AM *amconf, Jconf *jconf)
   if (jconf->input.type == INPUT_WAVEFORM) {
     /* Decode parameter extraction type according to the training
        parameter type in the header of the given acoustic HMM */
-    if ((hmminfo->opt.param_type & F_BASEMASK) != F_MFCC) {
-      jlog("ERROR: m_fusion: for direct speech input, only HMM trained by MFCC is supported\n");
+    switch(hmminfo->opt.param_type & F_BASEMASK) {
+    case F_MFCC:
+    case F_FBANK:
+    case F_MELSPEC:
+      break;
+    default:
+      jlog("ERROR: m_fusion: for direct speech input, only HMM trained by MFCC ior filterbank is supported\n");
       hmminfo_free(hmminfo);
       return NULL;
     }
@@ -236,8 +252,13 @@ initialize_GMM(Jconf *jconf)
   if (jconf->input.type == INPUT_WAVEFORM) {
     /* Decode parameter extraction type according to the training
        parameter type in the header of the given acoustic HMM */
-    if ((gmm->opt.param_type & F_BASEMASK) != F_MFCC) {
-      jlog("ERROR: m_fusion: for direct speech input, only GMM trained by MFCC is supported\n");
+    switch(gmm->opt.param_type & F_BASEMASK) {
+    case F_MFCC:
+    case F_FBANK:
+    case F_MELSPEC:
+      break;
+    default:
+      jlog("ERROR: m_fusion: for direct speech input, only GMM trained by MFCC ior filterbank is supported\n");
       hmminfo_free(gmm);
       return NULL;
     }
@@ -1317,6 +1338,11 @@ j_final_fusion(Recog *recog)
 	return FALSE;
       }
     }
+    /* when "-outprobout" is specified, ask the state computation
+       module to force calculatation of ALL the states at each
+       frame */
+    outprob_set_batch_computation(&(am->hmmwrk), (recog->jconf->outprob_outfile != NULL) ? TRUE : FALSE);
+
   }
 
   /* stage 5: initialize work area for input and realtime decoding */
@@ -1366,6 +1392,25 @@ j_final_fusion(Recog *recog)
     }
   }
 
+  /* initialize CMN and CVN calculation work area for batch computation */
+  if (! recog->jconf->decodeopt.realtime_flag) {
+    if (recog->jconf->input.type == INPUT_WAVEFORM) {
+      for(mfcc = recog->mfcclist; mfcc; mfcc = mfcc->next) {
+	if (mfcc->cmn.load_filename) {
+	  if (mfcc->para->cmn || mfcc->para->cvn) {
+	    mfcc->cmn.wrk = CMN_realtime_new(mfcc->para, mfcc->cmn.map_weight);
+	    if ((mfcc->cmn.loaded = CMN_load_from_file(mfcc->cmn.wrk, mfcc->cmn.load_filename))== FALSE) {
+	      jlog("ERROR: m_fusion: failed to read initial cepstral mean from \"%s\"\n", mfcc->cmn.load_filename);
+	      return FALSE;
+	    }
+	  } else {
+	    jlog("WARNING: m_fusion: CMN load file specified but AM not require it, ignored\n");
+	  }
+	}
+      }
+    }
+  }
+
   /* finished! */
   jlog("STAT: All init successfully done\n\n");
 
