@@ -111,7 +111,7 @@
  * @author Akinobu Lee
  * @date   Tue Aug 23 11:44:14 2005
  *
- * $Revision: 1.11 $
+ * $Revision: 1.13 $
  * 
  */
 /*
@@ -164,7 +164,7 @@ init_param(MFCCCalc *mfcc)
 
   /* 줫׻ѥ᡼ηإå */
   /* set header types */
-  mfcc->param->header.samptype = F_MFCC;
+  mfcc->param->header.samptype = para->basetype;
   if (para->delta) mfcc->param->header.samptype |= F_DELTA;
   if (para->acc) mfcc->param->header.samptype |= F_ACCL;
   if (para->energy) mfcc->param->header.samptype |= F_ENERGY;
@@ -817,7 +817,10 @@ RealTimePipeLine(SP16 *Speech, int nowlen, Recog *recog) /* Speech[0...nowlen] =
     /* Ĺ maxframelen ã餳Ƕλ */
     /* if input length reaches maximum buffer size, terminate 1st pass here */
     for (mfcc = recog->mfcclist; mfcc; mfcc = mfcc->next) {
-      if (mfcc->f >= r->maxframelen) return(1);
+      if (mfcc->f >= r->maxframelen) {
+	jlog("Warning: too long input (> %d frames), segment it now\n", r->maxframelen);
+	return(1);
+      }
     }
     /* Хåե */
     /* fill window buffer as many as possible */
