@@ -40,13 +40,13 @@
  * @author Akinobu Lee
  * @date   Sun Sep 18 19:40:34 2005
  *
- * $Revision: 1.4 $
+ * $Revision: 1.7 $
  * 
  */
 /*
- * Copyright (c) 1991-2012 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2013 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2012 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2013 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
@@ -142,8 +142,8 @@ wav2mfcc(SP16 speech[], int speechlen, Recog *recog)
     }
   
     /* make MFCC from speech data */
-    if (Wav2MFCC(speech, mfcc->param->parvec, para, speechlen, mfcc->wrk) == FALSE) {
-      jlog("ERROR: failed to compute MFCC from input speech\n");
+    if (Wav2MFCC(speech, mfcc->param->parvec, para, speechlen, mfcc->wrk, mfcc->cmn.wrk) == FALSE) {
+      jlog("ERROR: failed to compute features from input speech\n");
       if (mfcc->frontend.sscalc) {
 	free(mfcc->frontend.ssbuf);
 	mfcc->frontend.ssbuf = NULL;
