@@ -20,17 +20,17 @@
  * @author Akinobu LEE
  * @date   Wed Feb 16 16:52:24 2005
  *
- * $Revision: 1.19 $
+ * $Revision: 1.21 $
  * 
  */
 /*
- * Copyright (c) 1991-2012 Kawahara Lab., Kyoto University
+ * Copyright (c) 1991-2013 Kawahara Lab., Kyoto University
  * Copyright (c) 2000-2005 Shikano Lab., Nara Institute of Science and Technology
- * Copyright (c) 2005-2012 Julius project team, Nagoya Institute of Technology
+ * Copyright (c) 2005-2013 Julius project team, Nagoya Institute of Technology
  * All rights reserved
  */
 
-/* $Id: ngram_read_arpa.c,v 1.19 2012/07/27 08:44:57 sumomo Exp $ */
+/* $Id: ngram_read_arpa.c,v 1.21 2013/06/20 17:14:24 sumomo Exp $ */
 
 /* words should be alphabetically sorted */
 
@@ -113,6 +113,9 @@ set_unigram(FILE *fp, NGRAM_INFO *ndata)
 
   /* malloc name area */
   ndata->wname = (char **)mymalloc(sizeof(char *) * ndata->max_word_num);
+  for (nid = 0; nid < ndata->max_word_num; nid++) {
+    ndata->wname[nid] = NULL;
+  }
 
   /* malloc data area */
   //t->bgn_upper = t->bgn_lower = t->bgn = t->num = NULL;
